__title__ = 'hkube_python_wrapper'
__description__ = 'Hkube Python Wrapper'
__url__ = 'https://github.com/kube-HPC/python-wrapper.hkube'
__author__ = 'Hkube'
__author_email__ = 'hkube.dev@gmail.com'
__license__ = 'MIT'
__copyright__ = 'Copyright 2019 Hkube'
